<?php /** @noinspection ALL */

const SECONDS_TO_EXPIRE = 43200;
const WSSECURITY_UTILITY_1_0 = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd";
const WSSECURITY_SECEXT_1_0 = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd";
const WSS_X_509_TOKEN_PROFILE_1_0 = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3";
const WSS_SOAP_MSG_SEC_1_0_BASE_64_BIN = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary";
require __DIR__ . '/vendor/autoload.php';

use RobRichards\XMLSecLibs\XMLSecurityDSig;
use RobRichards\XMLSecLibs\XMLSecurityKey;

// Load the XML to be signed
$soapMessage = new DOMDocument();
$soapMessage->load('./napi-zaras-utemezes-v8.xml');

$signingCertificateFile = './keys/sign.pem';
$privateKeyOfSigningCertificate = './keys/private-sign-nopw.pem';

// Let the magic happen
$resultDoc = signXML($soapMessage, $signingCertificateFile, $privateKeyOfSigningCertificate);
//// Save the signed XML
$resultDoc->save('signed.xml');

$result = $soapMessage->saveXML();
$authCertificate = './keys/auth.pem';
$privateKeyOfAuthCertificate = './keys/private-auth-nopw.pem';
$caCertificate = './keys/ntaktstsagemhu.crt';
$endPointUrl = 'https://pms.sgmdev.hu/ntak/v8';

$response = sendXML($endPointUrl, $result, $authCertificate ,$privateKeyOfAuthCertificate, $caCertificate);

print "response:\n$response\n";

function signXML($doc, $signingCertificate, $privateKeyFile)
{
    print 'signing ...';

    // SOAP header element reference
    $headerElement = $doc->getElementsByTagNameNS('http://schemas.xmlsoap.org/soap/envelope/', 'Header')->item(0);

    // SOAP body element reference
    $bodyElement = $doc->getElementsByTagNameNS('http://schemas.xmlsoap.org/soap/envelope/', 'Body')->item(0);

    // ID attribute of the BinarySecurityToken element
    $binarySecurityTokenId = generateUniqueId('X509', 34);

    // Security element for the WSS
    $securityElement = createSecurityElement($doc, $signingCertificate, $binarySecurityTokenId);

    // Adding the previously created Security element to the SOAP header
    $headerElement->appendChild($securityElement);

    // Createing signature element and all the necessary belongings, using the 3rd party library
    $signatureElement = new XMLSecurityDSig();

    // set the ID attribute of the previously created Signature element
    $signatureElement->sigNode->setAttribute("Id", generateUniqueId("SIG", 34));

    // Define the canonical method of the signature
    $signatureElement->setCanonicalMethod(XMLSecurityDSig::EXC_C14N);
    $signatureElement->canonicalizeSignedInfo();

    // Create a reference to the Body element
    $signatureElement->addReference(
        $bodyElement,
        XMLSecurityDSig::SHA384,
        null,
        array('id_name' => 'Id', 'prefix_ns' => WSSECURITY_UTILITY_1_0)
    );

    // Create a timestamp and add it to the Security element
    $timestampElement = createTimestamp($doc);
    $securityElement->appendChild($timestampElement);

    // Create a reference to the Timestamp element
    $signatureElement->addReference(
        $timestampElement,
        XMLSecurityDSig::SHA384,
        null,
        array('id_name' => 'Id', 'prefix_ns' => WSSECURITY_UTILITY_1_0)
    );

    // Load the private key before the signing procedure
    $privateKey = new XMLSecurityKey(XMLSecurityKey::RSA_SHA256, array('type' => 'private'));
    $privateKey->loadKey($privateKeyFile, TRUE, FALSE);

    // Create the KeyInfo section
    $signatureElement->appendToKeyInfo(createTokenReference($signatureElement->sigNode, $binarySecurityTokenId));

    // Sign the necessary parts of the SOAP message
    $signatureElement->sign($privateKey, $securityElement);

    // Append the Signature section to the previously created Security element
    $signatureElement->appendSignature($securityElement);

    print " done\n";

    return $doc;
}

function createSecurityElement($document, $cert_file, $binarySecurityTokenId)
{
    $security = $document->createElementNS(WSSECURITY_SECEXT_1_0, "wsse:Security");
    $security->setAttribute("xmlns:wsu", WSSECURITY_UTILITY_1_0);

    $binarySecurityToken = $document->createElement("wsse:BinarySecurityToken", getCertificateAsString($cert_file));
    $binarySecurityToken->setAttribute("EncodingType", WSS_SOAP_MSG_SEC_1_0_BASE_64_BIN);
    $binarySecurityToken->setAttribute("ValueType", WSS_X_509_TOKEN_PROFILE_1_0);
    $binarySecurityToken->setAttribute("wsu:Id", $binarySecurityTokenId);

    $security->appendChild($binarySecurityToken);

    return $security;
}

function getCertificateAsString($cert_file)
{
    $cert_file_content = file_get_contents($cert_file);
    preg_match_all('/-----BEGIN CERTIFICATE-----([a-zA-Z0-9\n\+\/\=]+)-----END CERTIFICATE-----/', $cert_file_content, $cert_array);
    $cert = $cert_array[1];
    return trim(preg_replace('/\s+/', '', implode('', $cert)));

}

function generateUniqueId($prefix, $valLength)
{
    $result = '';
    $moduleLength = 40;
    $steps = round(($valLength / $moduleLength) + 0.5);

    for ($i = 0; $i < $steps; $i++) {
        $result .= sha1(uniqid() . md5(rand() . uniqid()));
    }

    return $prefix . '-' . substr($result, 0, $valLength);
}

function createTimestamp($document)
{
    $timestamp = $document->createElementNS(WSSECURITY_UTILITY_1_0, "wsu:Timestamp");
    $currentTime = time();
    $secondsToExpire = SECONDS_TO_EXPIRE;
    $created = gmdate("Y-m-d\TH:i:s", $currentTime) . 'Z';
    $expires = gmdate("Y-m-d\TH:i:s", $currentTime + $secondsToExpire) . 'Z';
    $createdElement = $document->createElementNS(WSSECURITY_UTILITY_1_0, "wsu:Created", $created);
    $expiresElement = $document->createElementNS(WSSECURITY_UTILITY_1_0, "wsu:Expires", $expires);
    $timestamp->appendChild($createdElement);
    $timestamp->appendChild($expiresElement);
    return $timestamp;
}

function createTokenReference($document, $binarySecurityTokenId)
{
    $baseDoc = $document->ownerDocument;
    $securityTokenReference = $baseDoc->createElementNS(WSSECURITY_SECEXT_1_0, 'SecurityTokenReference');
    $reference = $baseDoc->createElementNS(WSSECURITY_SECEXT_1_0, 'wsse:Reference');
    $reference->setAttribute('URI', "#$binarySecurityTokenId");
    $reference->setAttribute('ValueType', WSS_X_509_TOKEN_PROFILE_1_0);
    $securityTokenReference->appendChild($reference);
    return $securityTokenReference;
}

function sendXML($endPointUrl, $payload, $authCertificate, $authKey, $caCertificate, $debug = false) {
    print 'sending ...';

    $curlHandler = curl_init($endPointUrl);

    curl_setopt($curlHandler, CURLOPT_VERBOSE, $debug);

    // Authentikációs kulcshoz tartozó privát kulcs felolvasása:
    curl_setopt($curlHandler, CURLOPT_SSLKEY, $authKey);
    // curl_setopt($curlHandler, CURLOPT_SSLKEYPASSWD, ‘privat kulcs jelszava’);

    // szerver tanusítványának felolvasása
    curl_setopt($curlHandler, CURLOPT_CAINFO, $caCertificate);

    curl_setopt($curlHandler,CURLOPT_SSL_VERIFYPEER,0);

    curl_setopt($curlHandler, CURLOPT_CAPATH, $caCertificate);

    // Authentikációs tanusítvány felolvasása:
    curl_setopt($curlHandler, CURLOPT_SSLCERT, $authCertificate);

    // Activate HTTP Post
    curl_setopt($curlHandler, CURLOPT_POST, true);

    // Set the conent that needs to be sent
    curl_setopt($curlHandler, CURLOPT_POSTFIELDS, $payload);

    // the curl_exec method call should return with the response instead of printing it to the stdout
    curl_setopt($curlHandler, CURLOPT_RETURNTRANSFER, true);

    // setting the header
    curl_setopt($curlHandler, CURLOPT_HTTPHEADER, array('Content-Type: text/xml;charset=UTF-8'));
    curl_setopt($curlHandler, CURLOPT_HEADER, 1);

    // send the request
    $response = curl_exec($curlHandler);

    // print the error, if there is any
    if (curl_errno($curlHandler) <> 0) {
        print "\nerr: " . curl_error($curlHandler) . "\n";
    }

    curl_close($curlHandler);

    print " done\n";
    return $response;
}

